<div class="{{ widget }}" id="{{ widget }}">
	<div class="widgetIcons">
		<div class="widgetIcon">
			<i class="fas fa-calendar-alt" aria-hidden="true"></i>
		</div>
		<div class="widgetIcon" id="widgetSettings_{{ widget }}">
			<i class="fas fa-cog" aria-hidden="true"></i>
		</div>
	</div>
	<div>
		<!-- Your widget code -->
	</div>
</div>
